@@ -15,7 +15,6 @@
 
 from http import client as http_client
 
-from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import uuidutils
 from pecan import rest
@@ -29,6 +28,7 @@ from ironic.api import method
 from ironic.common import args
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states as ir_states
 from ironic import objects
 
@@ -46,14 +46,20 @@ PORT_SCHEMA = {
         'extra': {'type': ['object', 'null']},
         'is_smartnic': {'type': ['string', 'boolean', 'null']},
         'local_link_connection': {'type': ['null', 'object']},
+        'node_ident': {'type': 'string'},
         'node_uuid': {'type': 'string'},
         'physical_network': {'type': ['string', 'null'], 'maxLength': 64},
         'portgroup_uuid': {'type': ['string', 'null']},
         'pxe_enabled': {'type': ['string', 'boolean', 'null']},
         'uuid': {'type': ['string', 'null']},
         'name': {'type': ['string', 'null']},
+        'description': {'type': ['string', 'null'], 'maxLength': 255},
     },
-    'required': ['address', 'node_uuid'],
+    'required': ['address'],
+    'oneOf': [
+        {'required': ['node_ident']},
+        {'required': ['node_uuid']},
+    ],
     'additionalProperties': False,
 }
 
@@ -65,11 +71,13 @@ PATCH_ALLOWED_FIELDS = [
     'extra',
     'is_smartnic',
     'local_link_connection',
+    'node_ident',
     'node_uuid',
     'physical_network',
     'portgroup_uuid',
     'pxe_enabled',
     'name',
+    'description',
 ]
 
 PORT_VALIDATOR_EXTRA = args.dict_valid(
@@ -114,6 +122,23 @@ def hide_fields_in_newer_versions(port):
     # if requested version is < 1.88, hide name field.
     if not api_utils.allow_port_name():
         port.pop('name', None)
+    # note(JayF): if requested version is < 1.90, hide new
+    # local_link_connection schema but only check it if we allow advanced
+    # net fields, since otherwise we removed local_link_connection above
+    # and don't want to re-add it here
+    if (not api_utils.allow_ovn_vtep_version()
+            and api_utils.allow_port_advanced_net_fields):
+        local_link_connection = port.get('local_link_connection', {})
+        if any(key for key in local_link_connection.keys()
+               if key in api_utils.LOCAL_LINK_OVN_90_FIELDS):
+            # note(JayF): In this case, the field *should* exist but should be
+            # set to empty. This is because api version clients in this branch
+            # expect the key port.local_link_connection to exist even if we
+            # cannot set a valid value
+            port['local_link_connection'] = {}
+    # if requested version is < 1.97, hide description field.
+    if not api_utils.allow_port_description():
+        port.pop('description', None)
 
 
 def convert_with_links(rpc_port, fields=None, sanitize=True):
@@ -130,6 +155,7 @@ def convert_with_links(rpc_port, fields=None, sanitize=True):
             'pxe_enabled',
             'node_uuid',
             'name',
+            'description',
         )
     )
     if rpc_port.portgroup_id:
@@ -207,7 +233,8 @@ class PortsController(rest.RestController):
     def _get_ports_collection(self, node_ident, address, portgroup_ident,
                               shard, marker, limit, sort_key, sort_dir,
                               resource_url=None, fields=None, detail=None,
-                              project=None):
+                              project=None, description_contains=None,
+                              conductor_groups=None):
         """Retrieve a collection of ports.
 
         :param node_ident: UUID or name of a node, to get only ports for that
@@ -230,6 +257,10 @@ class PortsController(rest.RestController):
             of the resource to be returned.
         :param detail: Optional, show detailed list of ports
         :param project: Optional, filter by project
+        :param description_contains: Optional string value to get only ports
+                                     with description field contains matching
+                                     value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :returns: a list of ports.
 
         """
@@ -257,6 +288,15 @@ class PortsController(rest.RestController):
             if exclusive_filters > 1:
                 raise exception.OperationNotPermitted()
 
+        filters = {}
+        if description_contains:
+            filters['description_contains'] = description_contains
+
+        if conductor_groups and (node_ident or portgroup_ident or address):
+            raise exception.Invalid(
+                _("Filtering by conductor_groups cannot be combined with "
+                  "node_ident, portgroup_ident, or node address filters."))
+
         if portgroup_ident:
             # FIXME: Since all we need is the portgroup ID, we can
             #                 make this more efficient by only querying
@@ -268,7 +308,8 @@ class PortsController(rest.RestController):
                                                       marker_obj,
                                                       sort_key=sort_key,
                                                       sort_dir=sort_dir,
-                                                      project=project)
+                                                      project=project,
+                                                      filters=filters)
         elif node_ident:
             # FIXME(comstud): Since all we need is the node ID, we can
             #                 make this more efficient by only querying
@@ -279,18 +320,22 @@ class PortsController(rest.RestController):
                                                  node.id, limit, marker_obj,
                                                  sort_key=sort_key,
                                                  sort_dir=sort_dir,
-                                                 project=project)
+                                                 project=project,
+                                                 filters=filters)
         elif address:
             ports = self._get_ports_by_address(address, project=project)
         elif shard:
             ports = objects.Port.list_by_node_shards(api.request.context,
                                                      shard, limit,
                                                      marker_obj, sort_key,
-                                                     sort_dir, project=project)
+                                                     sort_dir, project=project,
+                                                     filters=filters)
         else:
             ports = objects.Port.list(api.request.context, limit,
                                       marker_obj, sort_key=sort_key,
-                                      sort_dir=sort_dir, project=project)
+                                      sort_dir=sort_dir, project=project,
+                                      conductor_groups=conductor_groups,
+                                      filters=filters)
         parameters = {}
 
         if detail is not None:
@@ -350,9 +395,16 @@ class PortsController(rest.RestController):
             if (not api_utils.allow_local_link_connection_network_type()
                     and 'network_type' in fields['local_link_connection']):
                 raise exception.NotAcceptable()
+            if (not api_utils.allow_ovn_vtep_version()
+                    and 'vtep-logical-switch'
+                    in fields['local_link_connection']):
+                raise exception.NotAcceptable()
         if ('name' in fields
                 and not api_utils.allow_port_name()):
             raise exception.NotAcceptable()
+        if ('description' in fields
+                and not api_utils.allow_port_description()):
+            raise exception.NotAcceptable()
 
     @METRICS.timer('PortsController.get_all')
     @method.expose()
@@ -361,10 +413,12 @@ class PortsController(rest.RestController):
                    limit=args.integer, sort_key=args.string,
                    sort_dir=args.string, fields=args.string_list,
                    portgroup=args.uuid_or_name, detail=args.boolean,
-                   shard=args.string_list)
+                   shard=args.string_list, description_contains=args.string,
+                   conductor_groups=args.string_list)
     def get_all(self, node=None, node_uuid=None, address=None, marker=None,
                 limit=None, sort_key='id', sort_dir='asc', fields=None,
-                portgroup=None, detail=None, shard=None):
+                portgroup=None, detail=None, shard=None,
+                description_contains=None, conductor_groups=None):
         """Retrieve a list of ports.
 
         Note that the 'node_uuid' interface is deprecated in favour
@@ -389,6 +443,10 @@ class PortsController(rest.RestController):
                                    for that portgroup.
         :param shard: Optional, a list of shard ids to filter by, only ports
                       associated with nodes in these shards will be returned.
+        :param description_contains: Optional string value to get only ports
+                                     with description field contains matching
+                                     value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :raises: NotAcceptable, HTTPNotFound
         """
         project = api_utils.check_port_list_policy(
@@ -413,6 +471,8 @@ class PortsController(rest.RestController):
         fields = api_utils.get_request_return_fields(fields, detail,
                                                      _DEFAULT_RETURN_FIELDS)
 
+        extra_args = {'description_contains': description_contains}
+
         if not node_uuid and node:
             # We're invoking this interface using positional notation, or
             # explicitly using 'node'.  Try and determine which one.
@@ -426,7 +486,9 @@ class PortsController(rest.RestController):
                                           sort_key, sort_dir,
                                           resource_url='ports',
                                           fields=fields, detail=detail,
-                                          project=project)
+                                          project=project,
+                                          conductor_groups=conductor_groups,
+                                          **extra_args)
 
     @METRICS.timer('PortsController.detail')
     @method.expose()
@@ -434,10 +496,11 @@ class PortsController(rest.RestController):
                    address=args.mac_address, marker=args.uuid,
                    limit=args.integer, sort_key=args.string,
                    sort_dir=args.string, portgroup=args.uuid_or_name,
-                   shard=args.string_list)
+                   shard=args.string_list, description_contains=args.string,
+                   conductor_groups=args.string_list)
     def detail(self, node=None, node_uuid=None, address=None, marker=None,
                limit=None, sort_key='id', sort_dir='asc', portgroup=None,
-               shard=None):
+               shard=None, description_contains=None, conductor_groups=None):
         """Retrieve a list of ports with detail.
 
         Note that the 'node_uuid' interface is deprecated in favour
@@ -460,6 +523,10 @@ class PortsController(rest.RestController):
                       max_limit resources will be returned.
         :param sort_key: column to sort results by. Default: id.
         :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
+        :param description_contains: Optional string value to get only ports
+                                    with description field contains matching
+                                    value.
+        :param conductor_groups: conductor groups, to filter the request by.
         :raises: NotAcceptable, HTTPNotFound
         """
         project = api_utils.check_port_list_policy(
@@ -485,11 +552,14 @@ class PortsController(rest.RestController):
         if parent != "ports":
             raise exception.HTTPNotFound()
 
+        extra_args = {'description_contains': description_contains}
         return self._get_ports_collection(node_uuid or node, address,
                                           portgroup, shard, marker, limit,
                                           sort_key, sort_dir,
                                           resource_url='ports/detail',
-                                          project=project)
+                                          project=project,
+                                          conductor_groups=conductor_groups,
+                                          **extra_args)
 
     @METRICS.timer('PortsController.get_one')
     @method.expose()
@@ -536,8 +606,17 @@ class PortsController(rest.RestController):
         node = None
         owner = None
         lessee = None
-        node_uuid = port.get('node_uuid')
+        node_uuid = port.get('node_uuid', None)
+        node_ident = port.get('node_ident', None)
+
+        if node_ident:
+            if not api_utils.allow_node_ident_as_param_for_port_creation():
+                raise exception.NotAcceptable()
+
+        ident = node_ident or node_uuid
         try:
+            node = api_utils.get_rpc_node(ident)
+            port['node_uuid'] = node['uuid']
             node = api_utils.replace_node_uuid_with_id(port)
             owner = node.owner
             lessee = node.lessee
@@ -681,7 +760,7 @@ class PortsController(rest.RestController):
                         context, port_dict['portgroup_uuid'])
                 else:
                     portgroup = None
-        except exception.PortGroupNotFound as e:
+        except exception.PortgroupNotFound as e:
             # Change error code because 404 (NotFound) is inappropriate
             # response for a PATCH request to change a Port
             e.code = http_client.BAD_REQUEST  # BadRequest
