@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import copy
 from http import client as http_client
 import inspect
@@ -103,8 +104,14 @@ LOCAL_LINK_BASE_SCHEMA = {
         'switch_info': {'type': 'string'},
         'network_type': {'type': 'string',
                          'enum': ['managed', 'unmanaged']},
+        'vtep-logical-switch': {'type': 'string'},
+        'vtep-physical-switch': {'type': 'string'},
     },
-    'additionalProperties': False
+    'additionalProperties': False,
+    'dependentRequired': {
+        'vtep-logical-switch': ['vtep-physical-switch'],
+        'vtep-physical-switch': ['vtep-logical-switch']
+    }
 }
 
 LOCAL_LINK_SCHEMA = copy.deepcopy(LOCAL_LINK_BASE_SCHEMA)
@@ -115,6 +122,11 @@ LOCAL_LINK_SMART_NIC_SCHEMA = copy.deepcopy(LOCAL_LINK_BASE_SCHEMA)
 # set mandatory fields for a smart nic
 LOCAL_LINK_SMART_NIC_SCHEMA['required'] = ['port_id', 'hostname']
 
+LOCAL_LINK_OVN_SCHEMA = copy.deepcopy(LOCAL_LINK_BASE_SCHEMA)
+LOCAL_LINK_OVN_SCHEMA['required'] = ['port_id', 'vtep-logical-switch',
+                                     'vtep-physical-switch']
+LOCAL_LINK_OVN_90_FIELDS = ['vtep-logical-switch', 'vtep-physical-switch']
+
 # no other mandatory fields for a network_type=unmanaged link
 LOCAL_LINK_UNMANAGED_SCHEMA = copy.deepcopy(LOCAL_LINK_BASE_SCHEMA)
 LOCAL_LINK_UNMANAGED_SCHEMA['properties']['network_type']['enum'] = [
@@ -125,6 +137,7 @@ LOCAL_LINK_CONN_SCHEMA = {'anyOf': [
     LOCAL_LINK_SCHEMA,
     LOCAL_LINK_SMART_NIC_SCHEMA,
     LOCAL_LINK_UNMANAGED_SCHEMA,
+    LOCAL_LINK_OVN_SCHEMA,
     {'type': 'object', 'additionalProperties': False},
 ]}
 
@@ -146,6 +159,24 @@ DEPLOY_STEP_SCHEMA = {
     'additionalProperties': False,
 }
 
+RUNBOOK_STEP_SCHEMA = {
+    'type': 'object',
+    'properties': {
+        'args': {'type': 'object'},
+        'interface': {
+            'type': 'string',
+            'enum': list(conductor_steps.CLEANING_INTERFACE_PRIORITY)
+        },
+        'step': {'type': 'string', 'minLength': 1},
+        'order': {'anyOf': [
+            {'type': 'integer', 'minimum': 0},
+            {'type': 'string', 'minLength': 1, 'pattern': '^[0-9]+$'}
+        ]}
+    },
+    'required': ['interface', 'step', 'order'],
+    'additionalProperties': False,
+}
+
 
 def local_link_normalize(name, value):
     if not value:
@@ -163,7 +194,7 @@ def local_link_normalize(name, value):
         except exception.InvalidDatapathID:
             raise exception.InvalidSwitchID(switch_id=value['switch_id'])
     except KeyError:
-        # In Smart NIC case 'switch_id' is optional.
+        # In Smart NIC or OVN VTEP case 'switch_id' is optional.
         pass
 
     return value
@@ -354,7 +385,7 @@ def patched_validate_with_schema(patched_dict, schema, validator=None):
         updates applied
     :param schema: Any dict key not in the schema will be deleted from the
         dict. If no validator is specified then the resulting ``patched_dict``
-        will be validated agains the schema
+        will be validated against the schema
     :param validator: Optional validator to use if there is extra validation
         required beyond the schema
     :raises: exception.Invalid if validation fails
@@ -368,6 +399,17 @@ def patched_validate_with_schema(patched_dict, schema, validator=None):
     validator('patch', patched_dict)
 
 
+def allow_special_characters_in_patch_fields():
+    """Check if special characters are allowed in patch field keys."""
+    return (api.request.version.minor
+            >= versions.MINOR_98_SUPPORT_SPECIAL_CHAR_IN_ATTRIBUTES)
+
+
+def decode_json_pointer(path):
+    """Decode JSON pointer as per RFC 6901 to support '~' and '/'."""
+    return path.replace('~1', '/').replace('~0', '~')
+
+
 def patch_validate_allowed_fields(patch, allowed_fields):
     """Validate that a patch list only modifies allowed fields.
 
@@ -379,7 +421,10 @@ def patch_validate_allowed_fields(patch, allowed_fields):
     """
     fields = set()
     for p in patch:
-        path = p['path'].split('/')[1]
+        if allow_special_characters_in_patch_fields():
+            path = decode_json_pointer(p['path'].split('/')[1])
+        else:
+            path = p['path'].split('/')[1]
         if path not in allowed_fields:
             msg = _("Cannot patch %s. Only the following can be updated: %s")
             raise exception.Invalid(
@@ -673,6 +718,43 @@ def get_rpc_deploy_template_with_suffix(template_ident):
                             exception.DeployTemplateNotFound)
 
 
+def get_rpc_runbook(runbook_ident):
+    """Get the RPC runbook from the UUID or logical name.
+
+    :param runbook_ident: the UUID or logical name of a runbook.
+
+    :returns: The RPC runbook.
+    :raises: InvalidUuidOrName if the name or uuid provided is not valid.
+    :raises: RunbookNotFound if the runbook is not found.
+    """
+    # If runbook_ident is instead a valid UUID, treat it as a UUID.
+    if uuidutils.is_uuid_like(runbook_ident):
+        return objects.Runbook.get_by_uuid(api.request.context,
+                                           runbook_ident)
+
+    # Else, we can refer to runbooks by their name too
+    if utils.is_valid_logical_name(runbook_ident):
+        return objects.Runbook.get_by_name(api.request.context,
+                                           runbook_ident)
+    raise exception.InvalidUuidOrName(name=runbook_ident)
+
+
+def check_runbook_policy_and_retrieve(policy_name, runbook_ident):
+    """Check if the specified policy authorizes this request on a node.
+
+    :param: policy_name: Name of the policy to check.
+    :param: runbook_ident: the UUID or logical name of a runbook.
+
+    :raises: HTTPForbidden if the policy forbids access.
+    :raises: RunbookNotFound if the runbook is not found.
+    :return: a runbook object
+    """
+    rpc_runbook = get_rpc_runbook(runbook_ident)
+    check_owner_policy(object_type='runbook', policy_name=policy_name,
+                       owner=rpc_runbook['owner'])
+    return rpc_runbook
+
+
 def is_valid_node_name(name):
     """Determine if the provided name is a valid node name.
 
@@ -809,7 +891,8 @@ VERSIONED_FIELDS = {
     'shard': versions.MINOR_82_NODE_SHARD,
     'parent_node': versions.MINOR_83_PARENT_CHILD_NODES,
     'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE,
-    'service_step': versions.MINOR_87_SERVICE
+    'service_step': versions.MINOR_87_SERVICE,
+    'disable_power_off': versions.MINOR_95_DISABLE_POWER_OFF,
 }
 
 for field in V31_FIELDS:
@@ -1505,6 +1588,61 @@ def check_policy_true(policy_name):
     return policy.check_policy(policy_name, cdict, api.request.context)
 
 
+def duplicate_steps(name, value):
+    """Argument validator to check template for duplicate steps"""
+    # TODO(mgoddard): Determine the consequences of allowing duplicate
+    # steps.
+    # * What if one step has zero priority and another non-zero?
+    # * What if a step that is enabled by default is included in a
+    #   template? Do we override the default or add a second invocation?
+
+    # Check for duplicate steps. Each interface/step combination can be
+    # specified at most once.
+    counter = collections.Counter((step['interface'], step['step'])
+                                  for step in value['steps'])
+    duplicates = {key for key, count in counter.items() if count > 1}
+    if duplicates:
+        duplicates = {"interface: %s, step: %s" % (interface, step)
+                      for interface, step in duplicates}
+        err = _("Duplicate deploy steps. A template cannot have multiple "
+                "deploy steps with the same interface and step. "
+                "Duplicates: %s") % "; ".join(duplicates)
+        raise exception.InvalidDeployTemplate(err=err)
+    return value
+
+
+def convert_steps(rpc_steps):
+    for step in rpc_steps:
+        result = {
+            'interface': step['interface'],
+            'step': step['step'],
+            'args': step['args'],
+        }
+
+        if 'priority' in step:
+            result['priority'] = step['priority']
+        elif 'order' in step:
+            result['order'] = step['order']
+
+        yield result
+
+
+def allow_runbooks():
+    """Check if accessing runbook endpoints is allowed.
+
+    Version 1.92 of the API exposed runbook endpoints.
+    """
+    return api.request.version.minor >= versions.MINOR_92_RUNBOOKS
+
+
+def allow_inspection_rules():
+    """Check if accessing inspection rule endpoints is allowed.
+
+    Version 1.96 of the API exposed rule endpoints.
+    """
+    return api.request.version.minor >= versions.MINOR_96_INSPECTION_RULES
+
+
 def check_owner_policy(object_type, policy_name, owner, lessee=None,
                        conceal_node=False):
     """Check if the policy authorizes this request on an object.
@@ -1535,6 +1673,19 @@ def check_owner_policy(object_type, policy_name, owner, lessee=None,
             raise
 
 
+def check_and_retrieve_public_runbook(runbook_ident):
+    """If policy authorization check fails, check if runbook is public.
+
+    :param: runbook_ident: the UUID or logical name of a runbook.
+    :raises: HTTPForbidden if runbook is not public.
+    :return: RPC runbook identified by runbook_ident
+    """
+    rpc_runbook = get_rpc_runbook(runbook_ident)
+    if not rpc_runbook.public:
+        raise exception.HTTPForbidden
+    return rpc_runbook
+
+
 def check_node_policy_and_retrieve(policy_name, node_ident,
                                    with_suffix=False):
     """Check if the specified policy authorizes this request on a node.
@@ -1623,6 +1774,27 @@ def check_multiple_node_policies_and_retrieve(policy_names,
     return rpc_node
 
 
+def check_multiple_runbook_policies_and_retrieve(policy_names,
+                                                 runbook_ident):
+    """Check if the specified policies authorize this request on a runbook.
+
+    :param: policy_names: List of policy names to check.
+    :param: runbook_ident: the UUID or logical name of a runbook.
+
+    :raises: HTTPForbidden if the policy forbids access.
+    :raises: RunbookNotFound if the runbook is not found.
+    :return: RPC runbook identified by runbook_ident
+    """
+    rpc_runbook = None
+    for policy_name in policy_names:
+        if rpc_runbook is None:
+            rpc_runbook = check_runbook_policy_and_retrieve(policy_names[0],
+                                                            runbook_ident)
+        else:
+            check_owner_policy('runbook', policy_name, rpc_runbook['owner'])
+    return rpc_runbook
+
+
 def check_list_policy(object_type, owner=None):
     """Check if the list policy authorizes this request on an object.
 
@@ -1887,6 +2059,15 @@ def check_volume_policy_and_retrieve(policy_name, vol_ident, target=False):
     return rpc_vol, rpc_node
 
 
+def allow_ovn_vtep_version():
+    """Check if ovn vtep version is allowed.
+
+    Version 1.90 of the API added support for ovn
+    vtep switches in port.local_link_connection.
+    """
+    return api.request.version.minor >= versions.MINOR_90_OVN_VTEP
+
+
 def allow_build_configdrive():
     """Check if building configdrive is allowed.
 
@@ -1985,6 +2166,22 @@ def check_allow_deploy_steps(target, deploy_steps):
             msg, status_code=http_client.BAD_REQUEST)
 
 
+def check_allow_boot_mode(node_capabilities, disallowed_boot_modes):
+    """Check if boot mode is allowed"""
+
+    if (not node_capabilities) or (not disallowed_boot_modes):
+        return
+
+    disallowed_set = set(disallowed_boot_modes)
+
+    for capability in node_capabilities:
+        for item in capability.lower().split(','):
+            key, value = item.split(':')
+            if key.strip() == 'boot_mode' and value.strip() in disallowed_set:
+                raise exception.BootModeNotAllowed(mode=value.strip(),
+                                                   op=_('provisioning'))
+
+
 def check_allow_clean_disable_ramdisk(target, disable_ramdisk):
     if disable_ramdisk is None:
         return
@@ -2035,3 +2232,21 @@ def allow_port_name():
 def allow_attach_detach_vmedia():
     """Check if we should support virtual media actions."""
     return api.request.version.minor >= versions.MINOR_89_ATTACH_DETACH_VMEDIA
+
+
+def allow_get_vmedia():
+    """Check if we should support get virtual media action."""
+    return api.request.version.minor >= versions.MINOR_93_GET_VMEDIA
+
+
+def allow_node_ident_as_param_for_port_creation():
+    """Check if 'node_ident' parameter is allowed for port creation."""
+    return api.request.version.minor >= versions.MINOR_94_PORT_NODENAME
+
+
+def allow_port_description():
+    """Check if description is allowed for ports.
+
+    Version 1.97 of the API added description field to the port object.
+    """
+    return api.request.version.minor >= versions.MINOR_97_PORT_DESCRIPTION
