@@ -30,10 +30,6 @@ from ironic import objects
 
 LOG = log.getLogger(__name__)
 
-# TODO(pas-ha) remove in Rocky, until then it is a default
-# for CONF.neutron.url in noauth case when endpoint_override is not set
-DEFAULT_NEUTRON_URL = 'http://%s:9696' % CONF.my_ip
-
 _NEUTRON_SESSION = None
 
 VNIC_BAREMETAL = 'baremetal'
@@ -145,16 +141,61 @@ def unbind_neutron_port(port_id, client=None, context=None, reset_mac=True):
         raise exception.NetworkError(msg)
 
 
-def update_port_address(port_id, address, context=None):
+def unbind_neutron_port_if_bound(port_id, client=None, context=None):
+    """Check and if bound, unbind a neutron port.
+
+    A wrapper around unbind_neutron_port which checks if the port is already
+    bound, and if so then triggers the port to be unbound. This is critical
+    early on if a user has pre-bound the port.
+
+    If the port is missing from Neutron, then a NetworkError exception will
+    also be raised because in this code path, we explicitly expect the port
+    to already exist and be accessible.
+
+    :param port_id: Neutron Port ID
+    :param client: Optional Neutron client.
+    :param context: request context
+    :type context: ironic.common.context.RequestContext
+    :raises: NetworkError
+    """
+    is_bound = None
+    if not client:
+        client = get_client(context=context)
+    try:
+        port = client.get_port(port_id)
+        if not (port.binding_host_id is None or port.binding_host_id == ''):
+            is_bound = True
+    except openstack_exc.ResourceNotFound:
+        msg = (_('The neutron port %(port_id)s was not found when we expect '
+                 'it to exist. We cannot proceed.') % {'port_id': port_id})
+        raise exception.NetworkError(msg)
+    except openstack_exc.OpenStackCloudException as e:
+        msg = (_('An unknown error was encountered while attempting '
+                 'to retrieve neutron port %(port_id)s. Error: '
+                 '%(err)s') % {'port_id': port_id, 'err': e})
+        raise exception.NetworkError(msg)
+    if is_bound:
+        # If bound, then trigger an unbind using the already created
+        # context and client.
+        LOG.debug('Attempting to unbind port %s because it is already '
+                  'bound in Neutron.', port_id)
+        # Don't use a client or overall context
+        unbind_neutron_port(port_id)
+
+
+def update_port_address(port_id, address, context=None, client=None):
     """Update a port's mac address.
 
     :param port_id: Neutron port id.
     :param address: new MAC address.
     :param context: request context
     :type context: ironic.common.context.RequestContext
+    :param client: A neutron client object.
     :raises: FailedToUpdateMacOnPort
     """
-    client = get_client(context=context)
+    if not client:
+        client = get_client(context=context)
+
     port_attrs = {'mac_address': address}
 
     try:
@@ -263,7 +304,8 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
     """
     client = get_client(context=task.context)
     node = task.node
-    add_all_ports = CONF.neutron.add_all_ports
+    pxe_capability = 'pxe_boot' in task.driver.boot.capabilities
+    add_all_ports = CONF.neutron.add_all_ports or not pxe_capability
 
     # If Security Groups are specified, verify that they exist
     _verify_security_groups(security_groups, client)
@@ -301,7 +343,7 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
     if not ports_to_create:
         pxe_enabled = 'PXE-enabled ' if not add_all_ports else ''
         raise exception.NetworkError(_(
-            "No available %(enabled)sports on node %(node)s.") %
+            "No available %(enabled)s ports on node %(node)s.") %
             {'enabled': pxe_enabled, 'node': node.uuid})
 
     for ironic_port in ports_to_create:
@@ -315,8 +357,20 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
             continue
 
         update_port_attrs['mac_address'] = ironic_port.address
+
+        # Stores local link information for the port
         binding_profile = {'local_link_information':
                            [portmap[ironic_port.uuid]]}
+
+        # Determine if network type is OVN
+        if is_ovn_vtep_port(ironic_port):
+            vtep_logical_switch = \
+                portmap[ironic_port.uuid]['vtep-logical-switch']
+            vtep_physical_switch = \
+                portmap[ironic_port.uuid]['vtep-physical-switch']
+            binding_profile['vtep-logical-switch'] = vtep_logical_switch
+            binding_profile['vtep-physical-switch'] = vtep_physical_switch
+
         update_port_attrs['binding:profile'] = binding_profile
 
         if not ironic_port.pxe_enabled:
@@ -353,15 +407,34 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
                                        update_port_attrs)
             if CONF.neutron.dhcpv6_stateful_address_count > 1:
                 _add_ip_addresses_for_ipv6_stateful(task.context, port, client)
-            if is_smart_nic:
-                wait_for_port_status(client, port.id, 'ACTIVE')
+
+            binding_fail_fatal = False
+            is_neutron_iface = node.network_interface == 'neutron'
+
+            if is_neutron_iface:
+                binding_fail_fatal = getattr(
+                    CONF.neutron, 'fail_on_port_binding_failure', False)
+
+            default_failure_behavior = is_smart_nic or binding_fail_fatal
+
+            fail_on_binding_failure = node.driver_info.get(
+                'fail_on_binding_failure', default_failure_behavior)
+
+            # NOTE(cid): Only check port status if it's a smart NIC or if we're
+            # configured to fail on binding failures. This avoids unnecessary
+            # failures when using network interfaces where binding may fail but
+            # we want to continue anyway (like in the case of flat networks).
+            if fail_on_binding_failure:
+                wait_for_port_status(
+                    client, port.id, 'ACTIVE',
+                    fail_on_binding_failure=fail_on_binding_failure)
         except openstack_exc.OpenStackCloudException as e:
             failures.append(ironic_port.uuid)
             LOG.warning("Could not create neutron port for node's "
-                        "%(node)s port %(ir-port)s on the neutron "
+                        "%(node)s port %(ir_port)s on the neutron "
                         "network %(net)s. %(exc)s",
                         {'net': network_uuid, 'node': node.uuid,
-                         'ir-port': ironic_port.uuid, 'exc': e})
+                         'ir_port': ironic_port.uuid, 'exc': e})
         else:
             ports[ironic_port.uuid] = port.id
 
@@ -384,6 +457,28 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
     return ports
 
 
+def is_ovn_vtep_port(port_info):
+    """Check if the current port is an OVN VTEP port
+
+    :param port_info: an instance of ironic.objects.port.Port
+        or port data as a port like object
+    :returns: Boolean indicating if the port is an OVN VTEP port
+    """
+
+    local_link_connection = {}
+
+    if isinstance(port_info, objects.Port):
+        local_link_connection = port_info.local_link_connection
+    elif isinstance(port_info, dict):
+        local_link_connection = port_info['local_link_connection']
+
+    if all(k in local_link_connection.keys()
+           for k in ['vtep-logical-switch', 'vtep-physical-switch']):
+        return True
+
+    return False
+
+
 def remove_ports_from_network(task, network_uuid):
     """Deletes the neutron ports created for booting the ramdisk.
 
@@ -466,17 +561,29 @@ def _uncidr(cidr, ipv6=False):
     return str(net.network_address), str(net.netmask)
 
 
-def get_neutron_port_data(port_id, vif_id, client=None, context=None):
+def get_neutron_port_data(port_id, vif_id, client=None, context=None,
+                          mac_address=None, iface_type='vif',
+                          bond_links=None):
     """Gather Neutron port and network configuration
 
     Query Neutron for port and network configuration, return whatever
-    is available.
+    is available. Apply a mac_address override, which may be useful for
+    the caller if this is being executed prior to port attachments.
 
     :param port_id: ironic port/portgroup ID.
     :param vif_id: Neutron port ID.
     :param client: Optional a Neutron client object.
     :param context: request context
     :type context: ironic.common.context.RequestContext
+    :param mac_address: An override ethernet mac address.
+    :param iface_type: Default interface type to denote a link
+                       to be. Likely ignored by cloud-init, but
+                       still best to be populated. Other applicable
+                       option is 'phy', or 'bond'.
+    :param bond_links: A list of dictionaries of additional links
+                       which indicate the port is a bind. This
+                       value being set overrides the iface_type
+                       value to 'bond'
     :raises: NetworkError
     :returns: a dict holding network configuration information
          associated with this ironic or Neutron port.
@@ -497,6 +604,11 @@ def get_neutron_port_data(port_id, vif_id, client=None, context=None):
     LOG.debug('Received port %(port)s data: %(info)s',
               {'port': vif_id, 'info': port_config})
 
+    # NOTE(TheJulia): We might want to change this to be just an ID
+    # at some point. The name is not a unique field, so someone evaluating
+    # the metadata may become confused. That being said, this also really
+    # is not hurting anyone and could be friendlier to a user as well.
+    # https://github.com/openstack/neutron/blob/ec34283b9d11924aaa8141cde23281753f5ecbcb/neutron/db/models_v2.py#L125
     port_id = port_config['name'] or port_id
 
     network_id = port_config.network_id
@@ -515,21 +627,48 @@ def get_neutron_port_data(port_id, vif_id, client=None, context=None):
 
     subnets_config = {}
 
+    if mac_address:
+        use_mac_address = mac_address
+    else:
+        use_mac_address = port_config['mac_address']
+
+    if bond_links:
+        # If we have bond links, we have to override the
+        # default value for the mapping.
+        iface_type = 'bond'
+
     network_data = {
         'links': [
             {
                 'id': port_id,
-                'type': 'vif',
-                'ethernet_mac_address': port_config['mac_address'],
+                'type': iface_type,
+                'ethernet_mac_address': use_mac_address,
                 'vif_id': port_config['id'],
                 'mtu': network_config['mtu']
             }
         ],
         'networks': [
 
+        ],
+
+        'services': [
+
         ]
     }
 
+    # Finish setup of bond links configuration injection
+    if bond_links:
+        links = []
+        for link in bond_links:
+            links.append(link['id'])
+        # Add the list of links to the bond port
+        network_data['links'][0]['bond_links'] = links
+        # Add the rest of the links to the metadata.
+        network_data['links'].extend(bond_links)
+
+    # Proceed with attempting to hydrate the rest of the
+    # network metadata around networks and services.
+
     for fixed_ip in port_config.get('fixed_ips', []):
         subnet_id = fixed_ip['subnet_id']
 
@@ -594,6 +733,18 @@ def get_neutron_port_data(port_id, vif_id, client=None, context=None):
 
         network_data['networks'].append(network)
 
+        for dns_nameserver in subnet_config['dns_nameservers']:
+            service = {
+                'type': 'dns',
+                'address': dns_nameserver
+            }
+            # NOTE(TheJulia): 'services' is expected to be defined on a
+            # network level and in the top level for the overall network
+            # data structure. Unknown if anyone actually uses it beyond
+            # just the top level. See this link:
+            # https://docs.openstack.org/nova/latest/_downloads/9119ca7ac90aa2990e762c08baea3a36/network_data.json  # noqa
+            network_data['services'].append(service)
+
     return network_data
 
 
@@ -692,7 +843,8 @@ def validate_network(uuid_or_name, net_type=_('network'), context=None):
     """
     if not uuid_or_name:
         raise exception.MissingParameterValue(
-            _('UUID or name of %s is not set in configuration') % net_type)
+            _('UUID or name of %s is not set in configuration or '
+              'in node driver_info.') % net_type)
 
     client = get_client(context=context)
     network = _get_network_by_uuid_or_name(client, uuid_or_name,
@@ -780,6 +932,9 @@ def is_smartnic_port(port_data):
     if isinstance(port_data, dict):
         return port_data.get('is_smartnic', False)
 
+    if isinstance(port_data, objects.Portgroup):
+        return False
+
     LOG.warning('Unknown port data type: %(type)s', {'type': type(port_data)})
     return False
 
@@ -838,11 +993,51 @@ def _get_port_by_uuid(client, port_uuid):
     return port
 
 
+def _get_segment_by_subnet_uuid(client, subnet_uuid):
+    """Return a neutron segment from a subnet UUID.
+
+    :param client: A Neutron client object.
+    :param subnet_uuid: UUID of a Neutron subnet to query.
+    :returns: A dict describing the neutron segment, or None.
+    :raises: InvalidParameterValue if the subnet or segment does not exist.
+    :raises: NetworkError on failure to contact Neutron.
+    """
+    try:
+        subnet = client.get_subnet(subnet_uuid)
+    except openstack_exc.ResourceNotFound:
+        raise exception.InvalidParameterValue(
+            _('Neutron subnet %(subnet_uuid)s was not found') %
+            {'subnet_uuid': subnet_uuid})
+    except openstack_exc.OpenStackCloudException as exc:
+        raise exception.NetworkError(
+            _('Could not retrieve neutron subnet: %s') %
+            exc)
+
+    segment_id = subnet.get('segment_id')
+    if segment_id is None:
+        return None
+
+    try:
+        segment = client.get_segment(segment_id)
+    except openstack_exc.ResourceNotFound:
+        raise exception.InvalidParameterValue(
+            _('Neutron segment %(segment_id)s was not found') %
+            {'segment_id': segment_id})
+    except openstack_exc.OpenStackCloudException as exc:
+        raise exception.NetworkError(
+            _('Could not retrieve neutron segment: %s') %
+            exc)
+
+    return segment
+
+
 def get_physnets_by_port_uuid(client, port_uuid):
     """Return the set of physical networks associated with a neutron port.
 
     Query the network to which the port is attached and return the set of
-    physical networks associated with the segments in that network.
+    physical networks associated with the segments in that network. If a port
+    is assigned to a subnet with a direct segment mapping, return the physnet
+    associated with its segment instead.
 
     :param client: A Neutron client object.
     :param port_uuid: UUID of a Neutron port to query.
@@ -856,6 +1051,20 @@ def get_physnets_by_port_uuid(client, port_uuid):
     network = _get_network_by_uuid_or_name(client, network_uuid)
 
     if network.segments is not None:
+        # If a port already has IP addresses assigned, this may indicate
+        # which segment it is connected to.
+        for fixed_ip in port.get('fixed_ips', []):
+            segment = _get_segment_by_subnet_uuid(client,
+                                                  fixed_ip.get('subnet_id'))
+            # When a network uses l2_adjancency, subnets don't map directly to
+            # segments
+            if segment is not None:
+                physnet = segment.get('physical_network')
+                # Return the segment's physnet if there is one for this network
+                # type
+                if physnet is not None:
+                    return set([physnet])
+
         # A network with multiple segments will have a 'segments' parameter
         # which will contain a list of segments. Each segment should have a
         # 'provider:physical_network' parameter which contains the physical
@@ -915,12 +1124,15 @@ def wait_for_host_agent(client, host_id, target_state='up'):
     stop=tenacity.stop_after_attempt(CONF.agent.neutron_agent_max_attempts),
     wait=tenacity.wait_fixed(CONF.agent.neutron_agent_status_retry_interval),
     reraise=True)
-def wait_for_port_status(client, port_id, status):
+def wait_for_port_status(client, port_id, status,
+                         fail_on_binding_failure=None):
     """Wait for port status to be the desired status
 
     :param client: A Neutron client object.
     :param port_id: Neutron port_id
     :param status: Port's target status, can be ACTIVE, DOWN ... etc.
+    :param fail_on_binding_failure: Whether to raise exception if port
+        binding fails.
     :returns: boolean indicates that the port status matches the
         required value passed by param status.
     :raises: InvalidParameterValue if the port does not exist.
@@ -934,6 +1146,14 @@ def wait_for_port_status(client, port_id, status):
               {'port_id': port_id, 'status': port.status})
     if port.status == status:
         return True
+
+    # fail early on port binding failure
+    if port.get('binding:vif_type') == 'binding_failed':
+        msg = "Binding failed for neutron port %s" % port_id
+        if fail_on_binding_failure:
+            LOG.error(msg)
+            raise openstack_exc.OpenStackCloudException(msg)
+        LOG.warning(msg)
     raise exception.NetworkError(
         'Port %(port_id)s failed to reach status %(status)s' % {
             'port_id': port_id, 'status': status})
@@ -947,7 +1167,7 @@ class NeutronNetworkInterfaceMixin(object):
             or CONF.neutron.cleaning_network
         )
         return validate_network(
-            cleaning_network, _('cleaning network'),
+            cleaning_network, 'cleaning_network',
             context=task.context)
 
     def get_provisioning_network_uuid(self, task):
@@ -956,7 +1176,7 @@ class NeutronNetworkInterfaceMixin(object):
             or CONF.neutron.provisioning_network
         )
         return validate_network(
-            provisioning_network, _('provisioning network'),
+            provisioning_network, 'provisioning_network',
             context=task.context)
 
     # TODO(stendulker): FlatNetwork should not use this method.
@@ -967,7 +1187,7 @@ class NeutronNetworkInterfaceMixin(object):
             or CONF.neutron.rescuing_network
         )
         return validate_network(
-            rescuing_network, _('rescuing network'),
+            rescuing_network, 'rescuing_network',
             context=task.context)
 
     def get_inspection_network_uuid(self, task):
@@ -976,7 +1196,7 @@ class NeutronNetworkInterfaceMixin(object):
             or CONF.neutron.inspection_network
         )
         return validate_network(
-            inspection_network, _('inspection network'),
+            inspection_network, 'inspection_network',
             context=task.context)
 
     def validate_inspection(self, task):
@@ -989,10 +1209,11 @@ class NeutronNetworkInterfaceMixin(object):
         """
         try:
             self.get_inspection_network_uuid(task)
-        except exception.MissingParameterValue:
+        except exception.MissingParameterValue as exc:
             # Fall back to non-managed in-band inspection
             raise exception.UnsupportedDriverExtension(
-                driver=task.node.driver, extension='inspection')
+                _("Insufficient information provided for managed "
+                  "inspection: %s") % exc)
 
     def get_servicing_network_uuid(self, task):
         servicing_network = (
