@@ -13,7 +13,6 @@
 #    under the License.
 
 import contextlib
-import crypt
 import datetime
 import functools
 import os
@@ -26,9 +25,11 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import excutils
+from oslo_utils import secretutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
 
+from ironic.common import async_steps
 from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common import faults
@@ -48,8 +49,8 @@ CONF = cfg.CONF
 
 
 PASSWORD_HASH_FORMAT = {
-    'sha256': crypt.METHOD_SHA256,
-    'sha512': crypt.METHOD_SHA512,
+    'sha256': 'SHA-256',
+    'sha512': 'SHA-512',
 }
 
 
@@ -289,13 +290,24 @@ def node_power_action(task, new_state, timeout=None):
              storage interface upon setting power on.
     :raises: other exceptions by the node's power driver if something
              wrong occurred during the power action.
-
+    :raises: ChildNodeLocked when a child node must be acted upon due
+             to this request, but we cannot act upon it due to a lock
+             being held.
+    :raises: ParentNodeLocked when a parent node must be acted upon
+             due to this request, but we cannot act upon it due to a
+             lock being held.
     """
     notify_utils.emit_power_set_notification(
         task, fields.NotificationLevel.INFO, fields.NotificationStatus.START,
         new_state)
     node = task.node
 
+    target_state = _calculate_target_state(new_state)
+    if target_state == states.POWER_OFF and node.disable_power_off:
+        LOG.error("Refusing to power off node %s with disable_power_off "
+                  "flag set", node.uuid)
+        raise exception.PowerStateFailure(pstate=new_state)
+
     if _can_skip_state_change(task, new_state):
         # NOTE(TheJulia): Even if we are not changing the power state,
         # we need to wipe the token out, just in case for some reason
@@ -305,7 +317,15 @@ def node_power_action(task, new_state, timeout=None):
             wipe_internal_info_on_power_off(node)
             node.save()
         return
-    target_state = _calculate_target_state(new_state)
+
+    # Parent node power required?
+    if task.node.parent_node:
+        # If we have a parent node defined for the device, we likely
+        # need to turn the power on.
+        parent_power_needed = not task.node.driver_info.get(
+            'has_dedicated_power_supply', False)
+    else:
+        parent_power_needed = False
 
     # Set the target_power_state and clear any last_error, if we're
     # starting a new operation. This will expose to other processes
@@ -330,6 +350,12 @@ def node_power_action(task, new_state, timeout=None):
             task.driver.storage.attach_volumes(task)
 
         if new_state != states.REBOOT:
+            if new_state == states.POWER_ON and parent_power_needed:
+                _handle_child_power_on(task, target_state, timeout)
+            if new_state == states.POWER_OFF:
+                _handle_child_power_off(task, target_state, timeout)
+            # Parent/Child dependencies handled, we can take care of
+            # the original node now.
             task.driver.power.set_power_state(task, new_state, timeout=timeout)
         else:
             # TODO(TheJulia): We likely ought to consider toggling
@@ -378,6 +404,68 @@ def node_power_action(task, new_state, timeout=None):
                             {'node': node.uuid, 'error': e})
 
 
+def _handle_child_power_on(task, target_state, timeout):
+    """Actions related to powering on a parent for a child node."""
+    with task_manager.acquire(
+            task.context, task.node.parent_node,
+            shared=True,
+            purpose='power on parent node') as pn_task:
+        if (pn_task.driver.power.get_power_state(pn_task)
+                != states.POWER_ON):
+            try:
+                pn_task.upgrade_lock()
+            except exception.NodeLocked as e:
+                LOG.error('Cannot power on parent_node %(pn)s '
+                          'as it is locked by %(host)s. We are '
+                          'unable to proceed with the current '
+                          'operation for child node %(node)s. '
+                          'Error: %(error)s',
+                          {'pn': pn_task.node.uuid,
+                           'host': pn_task.node.reservation,
+                           'node': task.node.uuid,
+                           'error': e})
+                raise exception.ParentNodeLocked(
+                    node=task.node.uuid, parent=pn_task.node.uuid)
+            node_power_action(pn_task, target_state, timeout)
+
+
+def _handle_child_power_off(task, target_state, timeout):
+    """Actions related to powering off child nodes."""
+    child_nodes = task.node.list_child_node_ids(exclude_dedicated_power=True)
+    cn_tasks = []
+    try:
+        for child in child_nodes:
+            # Get the task for the node, check the status, add
+            # the list of locks to act upon.
+            cn_task = task_manager.acquire(
+                task.context, child, shared=True,
+                purpose='child node power off')
+            if (cn_task.driver.power.get_power_state(cn_task)
+                != states.POWER_OFF):
+                cn_task.upgrade_lock()
+                cn_tasks.append(cn_task)
+        for cn_task in cn_tasks:
+            node_power_action(cn_task, target_state, timeout)
+    except exception.NodeLocked as e:
+        LOG.error(
+            'Cannot power off child node %(cn)s '
+            'as it is locked by %(host)s. We are '
+            'unable to proceed with the current '
+            'operation for parent node %(node)s. '
+            'Error: %(error)s',
+            {'cn': cn_task.node.uuid,
+             'host': cn_task.node.reservation,
+             'node': task.node.uuid,
+             'error': e})
+        raise exception.ChildNodeLocked(
+            node=task.node.uuid, child=cn_task.node.uuid)
+    finally:
+        for cn_task in cn_tasks:
+            # We don't need to do anything else with the child node,
+            # and we need to release the child task's lock.
+            cn_task.release_resources()
+
+
 @task_manager.require_exclusive_lock
 def cleanup_after_timeout(task):
     """Cleanup deploy task after timeout.
@@ -480,7 +568,10 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
             msg2 = ('Failed to tear down cleaning on node %(uuid)s, '
                     'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
             LOG.exception(msg2)
-            errmsg = _('%s. Also failed to tear down cleaning.') % errmsg
+            errmsg = _(
+                '%(orig_err)s. '
+                'Also %(msg2)s'
+            ) % {'orig_err': errmsg, 'msg2': msg2}
 
     if node.provision_state in (
             states.CLEANING,
@@ -490,9 +581,7 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
         node.clean_step = {}
         # Clear any leftover metadata about cleaning
         node.del_driver_internal_info('clean_step_index')
-        node.del_driver_internal_info('cleaning_reboot')
-        node.del_driver_internal_info('cleaning_polling')
-        node.del_driver_internal_info('skip_current_clean_step')
+        async_steps.remove_node_flags(node)
         # We don't need to keep the old agent URL, or token
         # as it should change upon the next cleaning attempt.
         wipe_token_and_url(task)
@@ -516,6 +605,25 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
         task.process_event('fail', target_state=target_state)
 
 
+def cleanup_servicewait_timeout(task):
+    """Cleanup a servicing task after timeout.
+
+    :param task: a TaskManager instance.
+    """
+    last_error = (_("Timeout reached while servicing the node. Please "
+                    "check if the ramdisk responsible for the servicing is "
+                    "running on the node. Failed on step %(step)s.") %
+                  {'step': task.node.service_step})
+    logmsg = ("Servicing for node %(node)s failed. %(error)s" %
+              {'node': task.node.uuid, 'error': last_error})
+    # NOTE(janders): this is called from the periodic task for servicewait
+    # timeouts, via the task manager's process_event(). The node has already
+    # been moved to SERVICEFAIL, so the error handler doesn't need to set the
+    # fail state.
+    servicing_error_handler(task, logmsg, errmsg=last_error,
+                            set_fail_state=False)
+
+
 def wipe_internal_info_on_power_off(node):
     """Wipe information that should not survive reboot/power off."""
     # DHCP may result in a new IP next time.
@@ -553,10 +661,9 @@ def wipe_deploy_internal_info(task):
     node.del_driver_internal_info('user_deploy_steps')
     node.del_driver_internal_info('agent_cached_deploy_steps')
     node.del_driver_internal_info('deploy_step_index')
-    node.del_driver_internal_info('deployment_reboot')
-    node.del_driver_internal_info('deployment_polling')
-    node.del_driver_internal_info('skip_current_deploy_step')
     node.del_driver_internal_info('steps_validated')
+    node.del_driver_internal_info('image_source')
+    async_steps.remove_node_flags(node)
 
 
 def wipe_cleaning_internal_info(task):
@@ -567,11 +674,22 @@ def wipe_cleaning_internal_info(task):
     node.set_driver_internal_info('clean_steps', None)
     node.del_driver_internal_info('agent_cached_clean_steps')
     node.del_driver_internal_info('clean_step_index')
-    node.del_driver_internal_info('cleaning_reboot')
-    node.del_driver_internal_info('cleaning_polling')
     node.del_driver_internal_info('cleaning_disable_ramdisk')
-    node.del_driver_internal_info('skip_current_clean_step')
     node.del_driver_internal_info('steps_validated')
+    async_steps.remove_node_flags(node)
+
+
+def wipe_service_internal_info(task):
+    """Remove temporary servicing fields from driver_internal_info."""
+    wipe_token_and_url(task)
+    node = task.node
+    node.set_driver_internal_info('service_steps', None)
+    node.del_driver_internal_info('agent_cached_service_steps')
+    node.del_driver_internal_info('service_step_index')
+    node.del_driver_internal_info('service_disable_ramdisk')
+    node.del_driver_internal_info('steps_validated')
+    node.del_driver_internal_info('image_source')
+    async_steps.remove_node_flags(node)
 
 
 def deploying_error_handler(task, logmsg, errmsg=None, traceback=False,
@@ -779,6 +897,11 @@ def spawn_deploying_error_handler(e, node):
     _spawn_error_handler(e, node, states.DEPLOYING)
 
 
+def spawn_servicing_error_handler(e, node):
+    """Handle spawning error for node servicing."""
+    _spawn_error_handler(e, node, states.SERVICING)
+
+
 def spawn_rescue_error_handler(e, node):
     """Handle spawning error for node rescue."""
     if isinstance(e, exception.NoFreeConductorWorker):
@@ -956,6 +1079,10 @@ def notify_conductor_resume_deploy(task):
     notify_conductor_resume_operation(task, 'deploy')
 
 
+def notify_conductor_resume_service(task):
+    notify_conductor_resume_operation(task, 'service')
+
+
 def skip_automated_cleaning(node):
     """Checks if node cleaning needs to be skipped for an specific node.
 
@@ -1037,10 +1164,11 @@ def power_state_for_network_configuration(task):
     restore_power_state_if_needed(task, previous)
 
 
+# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
 def build_configdrive(node, configdrive):
     """Build a configdrive from provided meta_data, network_data and user_data.
 
-    If uuid or name are not provided in the meta_data, they're defauled to the
+    If uuid or name are not provided in the meta_data, they're defaulted to the
     node's uuid and name accordingly.
 
     :param node: an Ironic node object.
@@ -1065,6 +1193,7 @@ def build_configdrive(node, configdrive):
                                 vendor_data=configdrive.get('vendor_data'))
 
 
+# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
 def get_configdrive_image(node):
     """Get configdrive as an ISO image or a URL.
 
@@ -1103,19 +1232,27 @@ def fast_track_able(task):
             # TODO(TheJulia): Should we check the provisioning/deployment
             # networks match config wise? Do we care? #decisionsdecisions
             and task.driver.storage.should_write_image(task)
-            and task.node.last_error is None)
+            and task.node.last_error is None
+            # NOTE(dtantsur): Fast track makes zero sense for servicing and
+            # may prevent normal clean-up from happening.
+            and task.node.provision_state not in states.SERVICING_STATES)
 
 
 def value_within_timeout(value, timeout):
     """Checks if the time is within the previous timeout seconds from now.
 
-    :param value: a string representing date and time or None.
+    :param value: a datetime or string representing date and time or None.
     :param timeout: timeout in seconds.
     """
     # use native datetime objects for conversion and compare
-    # slightly odd because py2 compatability :(
-    last = datetime.datetime.strptime(value or '1970-01-01T00:00:00.000000',
-                                      "%Y-%m-%dT%H:%M:%S.%f")
+    # slightly odd because py2 compatibility :(
+    if isinstance(value, datetime.datetime):
+        # Converts to a offset-naive datetime(as created by timeutils.utcnow())
+        last = value.replace(tzinfo=None)
+    else:
+        defaultdt = '1970-01-01T00:00:00.000000'
+        last = datetime.datetime.strptime(value or defaultdt,
+                                          '%Y-%m-%dT%H:%M:%S.%f')
     # If we found nothing, we assume that the time is essentially epoch.
     time_delta = datetime.timedelta(seconds=timeout)
     last_valid = timeutils.utcnow() - time_delta
@@ -1125,21 +1262,27 @@ def value_within_timeout(value, timeout):
 def agent_is_alive(node, timeout=None):
     """Check that the agent is likely alive.
 
-    The method then checks for the last agent heartbeat, and if it occured
+    The method then checks for the last agent heartbeat, and if it occurred
     within the timeout set by [deploy]fast_track_timeout, then agent is
     presumed alive.
 
     :param node: A node object.
     :param timeout: Heartbeat timeout, defaults to `fast_track_timeout`.
     """
+
+    timeout = timeout or CONF.deploy.fast_track_timeout
+    if node.power_state == states.POWER_ON and \
+       node.inspection_finished_at and \
+       value_within_timeout(node.inspection_finished_at, timeout):
+        return True
+
     # If no agent_url is present then we have powered down since the
     # last agent heartbeat
     if not node.driver_internal_info.get('agent_url'):
         return False
 
     return value_within_timeout(
-        node.driver_internal_info.get('agent_last_heartbeat'),
-        timeout or CONF.deploy.fast_track_timeout)
+        node.driver_internal_info.get('agent_last_heartbeat'), timeout)
 
 
 def is_fast_track(task):
@@ -1150,7 +1293,7 @@ def is_fast_track(task):
     have a ramdisk running through another means like discovery.
     If not valid, False is returned.
 
-    The method then checks for the last agent heartbeat, and if it occured
+    The method then checks for the last agent heartbeat, and if it occurred
     within the timeout set by [deploy]fast_track_timeout and the power
     state for the machine is POWER_ON, then fast track is permitted.
 
@@ -1209,7 +1352,7 @@ def _get_node_next_steps(task, step_type, skip_current_step=True):
     :returns: index of the next step; None if there are none to execute.
 
     """
-    valid_types = set(['clean', 'deploy'])
+    valid_types = set(['clean', 'deploy', 'service'])
     if step_type not in valid_types:
         # NOTE(rloo): No need to i18n this, since this would be a
         # developer error; it isn't user-facing.
@@ -1251,14 +1394,8 @@ def update_next_step_index(task, step_type):
     :param step_type: The type of steps to process: 'clean' or 'deploy'.
     :returns: Index of the next step.
     """
-    skip_current_step = task.node.del_driver_internal_info(
-        'skip_current_%s_step' % step_type, True)
-    if step_type == 'clean':
-        task.node.del_driver_internal_info('cleaning_polling')
-    else:
-        task.node.del_driver_internal_info('deployment_polling')
-    task.node.save()
-
+    skip_current_step = async_steps.prepare_node_for_next_step(
+        task.node, step_type)
     return _get_node_next_steps(task, step_type,
                                 skip_current_step=skip_current_step)
 
@@ -1322,7 +1459,7 @@ def is_agent_token_pregenerated(node):
 
     This method helps us identify WHEN we did so as we don't need to remove
     records of the token prior to rebooting the token. This is important as
-    tokens provided through out of band means presist in the virtual media
+    tokens provided through out of band means persist in the virtual media
     image, are loaded as part of the agent ramdisk, and do not require
     regeneration of the token upon the initial lookup, ultimately making
     the overall usage of virtual media and pregenerated tokens far more
@@ -1342,7 +1479,7 @@ def make_salt():
 
     :returns: a valid salt for use with crypt.crypt
     """
-    return crypt.mksalt(
+    return secretutils.crypt_mksalt(
         method=PASSWORD_HASH_FORMAT[
             CONF.conductor.rescue_password_hash_algorithm])
 
@@ -1352,7 +1489,7 @@ def hash_password(password=''):
 
     :param password: password to be hashed
     """
-    return crypt.crypt(password, make_salt())
+    return secretutils.crypt_password(password, make_salt())
 
 
 def get_attached_vif(port):
@@ -1427,18 +1564,18 @@ def store_agent_certificate(node, agent_verify_ca):
         return fname
 
 
-def node_cache_bios_settings(task, node):
+def node_cache_bios_settings(task):
     """Do caching of bios settings if supported by driver"""
     try:
-        LOG.debug('Getting BIOS info for node %s', node.uuid)
+        LOG.debug('Getting BIOS info for node %s', task.node.uuid)
         task.driver.bios.cache_bios_settings(task)
     except exception.UnsupportedDriverExtension:
         LOG.warning('BIOS settings are not supported for node %s, '
-                    'skipping', node.uuid)
-    # TODO(zshi) remove this check when classic drivers are removed
+                    'skipping', task.node.uuid)
     except Exception:
+        # NOTE(dtantsur): the caller expects this function to never fail
         msg = (_('Caching of bios settings failed on node %(node)s.')
-               % {'node': node.uuid})
+               % {'node': task.node.uuid})
         LOG.exception(msg)
 
 
@@ -1460,6 +1597,7 @@ def node_cache_vendor(task):
     except exception.UnsupportedDriverExtension:
         return
     except Exception as exc:
+        # NOTE(dtantsur): the caller expects this function to never fail
         LOG.warning('Unexpected exception when trying to detect vendor '
                     'for node %(node)s. %(class)s: %(exc)s',
                     {'node': task.node.uuid,
@@ -1635,7 +1773,7 @@ def node_history_record(node, conductor=None, event=None,
                        based upon the activity. The purpose is to help guide
                        an API consumer/operator to have a better contextual
                        understanding of what was going on *when* the "event"
-                       occured.
+                       occurred.
     :param user: The user_id value which triggered the request,
                  if available.
     :param error: Boolean value, default false, to signify if the event
@@ -1644,11 +1782,11 @@ def node_history_record(node, conductor=None, event=None,
     :returns: None. No value is returned by this method.
     """
     if not event:
-        # No error has occured, apparently.
+        # No error has occurred, apparently.
         return
     if error:
         # When the task exits out or is saved, the event
-        # or error is saved, but that is outside of ceating an
+        # or error is saved, but that is outside of creating an
         # entry in the history table.
         node.last_error = event
     if not conductor:
@@ -1688,7 +1826,7 @@ def update_image_type(context, node):
         # idea since it is also user-settable, but laregely is just geared
         # to take what is in glance. Line below should we wish to uncomment.
         # node.set_instance_info('image_type', images.IMAGE_TYPE_DIRECTORY)
-        # An alternative is to explictly allow it to be configured by the
+        # An alternative is to explicitly allow it to be configured by the
         # caller/requester.
         return True
 
@@ -1729,7 +1867,7 @@ def get_token_project_from_request(ctx):
     This method evaluates the ``auth_token_info`` field, which is used to
     pass information returned from keystone as a token's
     verification. This information is based upon the actual, original
-    requestor context provided ``auth_token``.
+    requester context provided ``auth_token``.
 
     When a service, such as Nova proxies a request, the request provided
     auth token value is intended to be from the original user.
@@ -1743,5 +1881,129 @@ def get_token_project_from_request(ctx):
             if project:
                 return project.get('id')
     except AttributeError:
-        LOG.warning('Attempted to identify requestor project ID value, '
+        LOG.warning('Attempted to identify requester project ID value, '
                     'however we were unable to do so. Possible older API?')
+
+
+def servicing_error_handler(task, logmsg, errmsg=None, traceback=False,
+                            tear_down_service=True, set_fail_state=True,
+                            set_maintenance=None):
+    """Put a failed node in SERVICEFAIL and maintenance (if needed).
+
+    :param task: a TaskManager instance.
+    :param logmsg: Message to be logged.
+    :param errmsg: Message for the user. Optional, if not provided `logmsg` is
+        used.
+    :param traceback: Whether to log a traceback. Defaults to False.
+    :param tear_down_service: Whether to clean up the PXE and DHCP files after
+        service. Default to True.
+    :param set_fail_state: Whether to set node to failed state. Default to
+        True.
+    :param set_maintenance: Whether to set maintenance mode. If None,
+        maintenance mode will be set if and only if a clean step is being
+        executed on a node.
+    """
+    if set_maintenance is None:
+        set_maintenance = bool(task.node.service_step)
+
+    errmsg = errmsg or logmsg
+    LOG.error(logmsg, exc_info=traceback)
+    node = task.node
+    if set_maintenance:
+        node.fault = faults.SERVICE_FAILURE
+        node.maintenance = True
+
+    if tear_down_service:
+        try:
+            task.driver.deploy.tear_down_service(task)
+        except Exception as e:
+            msg2 = ('Failed to tear down servicing on node %(uuid)s, '
+                    'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
+            LOG.exception(msg2)
+            errmsg = _('%s. Also failed to tear down servicing.') % errmsg
+
+    if node.provision_state in (
+            states.SERVICING,
+            states.SERVICEWAIT,
+            states.SERVICEFAIL):
+        # Clear clean step, msg should already include current step
+        node.service_step = {}
+        # Clear any leftover metadata about cleaning
+        node.del_driver_internal_info('service_step_index')
+        async_steps.remove_node_flags(node)
+        # We don't need to keep the old agent URL, or token
+        # as it should change upon the next cleaning attempt.
+        wipe_token_and_url(task)
+    # For manual cleaning, the target provision state is MANAGEABLE, whereas
+    # for automated cleaning, it is AVAILABLE.
+    node_history_record(node, event=errmsg, event_type=states.SERVICING,
+                        error=True)
+    # NOTE(dtantsur): avoid overwriting existing maintenance_reason
+    if not node.maintenance_reason and set_maintenance:
+        node.maintenance_reason = errmsg
+
+    if CONF.conductor.poweroff_in_servicefail:
+        # NOTE(NobodyCam): Power off node in service fail
+        node_power_action(task, states.POWER_OFF)
+
+    node.save()
+
+    if set_fail_state and node.provision_state != states.SERVICEFAIL:
+        task.process_event('fail')
+
+
+def node_cache_firmware_components(task):
+    """Do caching of firmware components if supported by driver"""
+
+    try:
+        LOG.debug('Getting Firmware Components for node %s', task.node.uuid)
+        task.driver.firmware.validate(task)
+        task.driver.firmware.cache_firmware_components(task)
+    except exception.UnsupportedDriverExtension:
+        LOG.warning('Firmware Components are not supported for node %s, '
+                    'skipping', task.node.uuid)
+    except Exception:
+        # NOTE(dtantsur): the caller expects this function to never fail
+        LOG.exception('Caching of firmware components failed on node %s',
+                      task.node.uuid)
+
+
+def run_node_action(task, call, error_msg, success_msg=None, **kwargs):
+    """Run a node action and report any errors via last_error.
+
+    :param task: A TaskManager instance containing the node to act on.
+    :param call: A callable object to invoke.
+    :param error_msg: A template for a failure message. Can use %(node)s,
+        %(exc)s and any variables from kwargs.
+    :param success_msg: A template for a success message. Can use %(node)s
+        and any variables from kwargs.
+    :param kwargs: Arguments to pass to the call.
+    """
+    error = None
+    try:
+        call(task, **kwargs)
+    except Exception as exc:
+        error = error_msg % dict(kwargs, node=task.node.uuid, exc=exc)
+        node_history_record(task.node, event=error, error=True)
+        LOG.error(
+            error, exc_info=not isinstance(exc, exception.IronicException))
+
+    task.node.save()
+    if not error and success_msg:
+        LOG.info(success_msg, dict(kwargs, node=task.node.uuid))
+
+
+def node_update_cache(task):
+    """Updates various cached information.
+
+    Includes vendor, boot mode, BIOS settings and firmware components.
+
+    :param task: A TaskManager instance containing the node to act on.
+    """
+    # FIXME(dtantsur): in case of Redfish, these 4 calls may result in the
+    # System object loaded at least 4 times. "Cache whatever you can" should
+    # probably be a driver call, just not clear in which interface.
+    node_cache_vendor(task)
+    node_cache_boot_mode(task)
+    node_cache_bios_settings(task)
+    node_cache_firmware_components(task)
