@@ -20,8 +20,6 @@ import os
 import shlex
 from urllib import parse as urlparse
 
-from ironic_lib import metrics_utils
-from ironic_lib import utils as irlib_utils
 from oslo_concurrency import processutils
 from oslo_log import log
 from oslo_utils import strutils
@@ -34,6 +32,7 @@ from ironic.common import exception
 from ironic.common import faults
 from ironic.common.i18n import _
 from ironic.common import images
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import steps as conductor_steps
@@ -381,11 +380,7 @@ class AnsibleDeploy(agent_base.HeartbeatMixin,
                     base.DeployInterface):
     """Interface for deploy-related actions."""
 
-    def __init__(self):
-        super(AnsibleDeploy, self).__init__()
-        # NOTE(pas-ha) overriding agent creation as we won't be
-        # communicating with it, only processing heartbeats
-        self._client = None
+    collect_deploy_logs = False
 
     def get_properties(self):
         """Return the properties of the interface."""
@@ -400,12 +395,6 @@ class AnsibleDeploy(agent_base.HeartbeatMixin,
         task.driver.boot.validate(task)
 
         node = task.node
-        iwdi = node.driver_internal_info.get('is_whole_disk_image')
-        if not iwdi and deploy_utils.get_boot_option(node) == "netboot":
-            raise exception.InvalidParameterValue(_(
-                "Node %(node)s is configured to use the ansible deploy "
-                "interface, which does not support netboot.") %
-                {'node': node.uuid})
 
         params = {}
         image_source = node.instance_info.get('image_source')
@@ -513,8 +502,7 @@ class AnsibleDeploy(agent_base.HeartbeatMixin,
             node.instance_info = deploy_utils.build_instance_info_for_deploy(
                 task)
             node.save()
-            boot_opt = deploy_utils.build_agent_options(node)
-            task.driver.boot.prepare_ramdisk(task, boot_opt)
+            deploy_utils.prepare_agent_boot(task)
 
     @METRICS.timer('AnsibleDeploy.clean_up')
     def clean_up(self, task):
@@ -522,8 +510,7 @@ class AnsibleDeploy(agent_base.HeartbeatMixin,
         task.driver.boot.clean_up_ramdisk(task)
         provider = dhcp_factory.DHCPFactory()
         provider.clean_dhcp(task)
-        irlib_utils.unlink_without_raise(
-            _get_configdrive_path(task.node.uuid))
+        utils.unlink_without_raise(_get_configdrive_path(task.node.uuid))
 
     def take_over(self, task):
         LOG.error("Ansible deploy does not support take over. "
@@ -599,8 +586,7 @@ class AnsibleDeploy(agent_base.HeartbeatMixin,
         task.driver.network.add_cleaning_network(task)
         manager_utils.restore_power_state_if_needed(
             task, power_state_to_restore)
-        boot_opt = deploy_utils.build_agent_options(node)
-        task.driver.boot.prepare_ramdisk(task, boot_opt)
+        deploy_utils.prepare_agent_boot(task)
         if not fast_track:
             manager_utils.node_power_action(task, states.REBOOT)
         return states.CLEANWAIT
